/*
	Monthly.js
	Logic for Monthly dashboard widget
	http://www.sorddin.com/software/monthly/

	Copyright (c) 2006, David Robinson <david@sorddin.com>
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this
	list of conditions and the following disclaimer.

	Redistributions in binary form must reproduce the above copyright notice, this
	list of conditions and the following disclaimer in the documentation and/or
	other materials provided with the distribution.

	The names "Sorddin" and "David Robinson" must not be used to endorse or 
	promote products derived from this software without prior written permission.
	For written permission, please contact david@sorddin.com.
 
	THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, 
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND 
	FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
	COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
	LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
	OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
	LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
	NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
	EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// Version number. If you change this, also change Info.plist

var gVersion = '1.2.1';


// Website for latest version

var gWebSite = 'http://www.sorddin.com/software/monthly/';


// Buttons

var gDoneButton;
var gInfoButton;


// Date holders

var gDate = new Date ();
var gPreviousDate = 0;
var gCurrentMonth = gDate.getMonth ();
var gCurrentYear = gDate.getFullYear ();
var gCurrentDate = gDate.getDate ();
var gCurrentHour = gDate.getHours ();
var gDays = null;


// Timer for daily updates

var timerInterval = null;


// current percentage of month

var mPercent = 0;
var mPercentOld = null;
var yearPercent = null;


// Preferances

var countUp = new Boolean(true);
var updateHourly = new Boolean(true); // if true then widget updates once and hour, if false widget updates one a day
var dayTicks = new Boolean(true);
var percentTicks = new Boolean(true);
var aniOn = new Boolean(true); // if true display nice animations when drawing arc

// Widget size details

var mHsize = 172;
var mVsize = mHsize; // the widget is square, this is just here for possible future use
var mHcen = mHsize/2;
var mVcen = mVsize/2;

var arcWidth = 10;
var arcInner = 90;
var arcSmall = arcInner/2;
var arcBig = arcSmall + arcWidth ;
	
var pillLenght = 100;
var pillHeight = 14;
var pillDist = pillHeight/2;
var pillCen = (pillLenght/2) - pillDist;


// Load clock tick marks and pill images

var mPill = new Image (mHsize, mVsize);
mPill.src = 'Images/pill.png';
var mPillUnder = new Image (mHsize, mVsize);
mPillUnder.src = 'Images/pillunder.png';
var mPer = new Image (mHsize, mVsize);
mPer.src = 'Images/percent.png';
var m28 = new Image (mHsize, mVsize);
m28.src = 'Images/28.png';
var m29 = new Image (mHsize, mVsize);
m29.src = 'Images/29.png';
var m30 = new Image (mHsize, mVsize);
m30.src = 'Images/30.png';
var m31 = new Image (mHsize, mVsize);
m31.src = 'Images/31.png';
var mTicks = new Image (mHsize, mVsize);



// Main routines
//

function onLoaded()
{
	// Display Info Button
	gDoneButton = new AppleGlassButton(document.getElementById('doneButton'), 'Done', hidePrefs);
	gInfoButton = new AppleInfoButton(document.getElementById('infoButton'), document.getElementById('front'), 'white', 'white', showPrefs);
	
	// Load preferences
	if (window.widget) {
	
		var checkPref;
		
		// init count direction prefs
		checkPref = widget.preferenceForKey(createkey('pref_countUp'));
		if (checkPref == 'countDown') {
			countUp = false;
			document.getElementById ('count-popup').options[1].selected = true;						
		}
		else {
			countUp = true;
			document.getElementById ('count-popup').options[0].selected = true;
		}
		
		// init hourly update prefs
		checkPref = widget.preferenceForKey(createkey('pref_updateFreq'));
		if (checkPref == 'daily') {
			updateHourly = false;
			document.getElementById ('update-popup').options[1].selected = true;						
		}
		else {
			updateHourly = true;
			document.getElementById ('update-popup').options[0].selected = true;
		}
		
		// init percent tick dispaly prefs
		checkPref = widget.preferenceForKey(createkey('pref_percentTicks'));
		if (checkPref == 'off') {
			percentTicks = false;
		}
		else {
			percentTicks = true;
		}
		document.getElementById('percent-checkbox').checked = percentTicks;
		
		// init days tick dispaly prefs
		checkPref = widget.preferenceForKey(createkey('pref_dayTicks'));
		if (checkPref == 'off') {
			dayTicks = false;
		}
		else {
			dayTicks = true;
		}
		document.getElementById('days-checkbox').checked = dayTicks;        

		// init animation dispaly prefs
		checkPref = widget.preferenceForKey(createkey('pref_aniOn'));
		if (checkPref == 'off') {
			aniOn = false;
		}
		else {
			aniOn = true;
		}
		document.getElementById('animate-checkbox').checked = aniOn;      
	}

	// Update display
	updateDate(true);
}



if (window.widget)
// Register for notifications
{
	widget.onhide = onhide;
	widget.onshow = onshow;
	widget.onremove = onremove;
}



function onshow(doSweepBack)
// update and pass animation through
{	
	updateDate(doSweepBack);
}



function onhide()
// when hidden there is no need to update the display, so clean up the timer
{
	if (timerInterval != null) {
		clearTimeout(timerInterval);
		timerInterval = null;
	}
}



function onremove()
// clean up and save prefs when closeing
{
	if (timerInterval != null) {
		clearTimeout(timerInterval);
		timerInterval = null;
	}
	
	widget.setPreferenceForKey (null, createkey('pref_countUp'));
	widget.setPreferenceForKey (null, createkey('pref_dayTicks'));
	widget.setPreferenceForKey (null, createkey('pref_updateFreq'));
	widget.setPreferenceForKey (null, createkey('pref_percentTicks'));
	widget.setPreferenceForKey (null, createkey('pref_aniOn'));
}



/*
Prefs pane functions
*/

function showPrefs()
{
    var front = document.getElementById('front');
    var back = document.getElementById('back');

    if (window.widget)
        widget.prepareForTransition('ToBack');
    onhide();

    front.style.display='none';
    back.style.display='block';

    if (window.widget)
    {
        setTimeout ('widget.performTransition(); widget.setCloseBoxOffset(12, 12);', 0);
    }
}



function hidePrefs()
{
    var front = document.getElementById('front');
    var back = document.getElementById('back');

    if (window.widget)
        widget.prepareForTransition('ToFront');
 
    front.style.display='block';
    back.style.display='none';
   
	setTimeout ('flipitback();', 0);    
}



function flipitback()
{
	onshow(false);
	if (window.widget) {
		setTimeout('widget.performTransition(); widget.setCloseBoxOffset(46, 46);', 0);
	}
}



/*
Update functions
*/

function startUpdateTimer()
{
	if (timerInterval != null) {
		clearTimeout (timerInterval);
		timerInterval = null;
	}

	// create a new timer for when we need to update the date
	var hourMillis = ((23 - gDate.getHours()) * 3600000);
	var minsMillis = ((59 - gDate.getMinutes()) * 60000);
	var secsMillis = ((60 - gDate.getSeconds()) * 1000); // overshoot a second
	
	var millisRemaining = hourMillis + minsMillis + secsMillis;
	if (updateHourly == true) {
		var millisRemaining = minsMillis + secsMillis;
	}
	
	timerInterval = setTimeout('updateDate(true);', millisRemaining);	
}



function updateDate(doSweepBack)
{
	if (doSweepBack == null) {
		doSweepBack = false;
	}
	if (mPercentOld == null) {
		mPercentOld = 0.0;
	}
	
	gDate = new Date();
	gCurrentMonth = gDate.getMonth ();
	gCurrentYear = gDate.getFullYear ();
	gCurrentDate = gDate.getDate ();
	gCurrentHour = gDate.getHours();
	gDays = calculateNumberOfDaysInMonth(gCurrentYear, gCurrentMonth);
	
	if (gCurrentDate != gPreviousDate  && updateHourly == false) {
		mPercentOld = mPercent;
		gPreviousDate = gCurrentDate;
		doSweepBack = true;
	}

	mPercent = gCurrentDate / gDays;
	if (updateHourly == true) {
		mPercent = (((gCurrentDate - 1) * 24) + gCurrentHour) / (gDays * 24);
	}
	
	yearPercent = gCurrentMonth / 11;
	
	switch (gDays) {
		case 28:    mTicks = m28;	break;
		case 29:    mTicks = m29;	break;
		case 30:    mTicks = m30;	break;
		case 31:    mTicks = m31;	break;
	}
	
	if (dayTicks == true || percentTicks == true) {
		drawTicks();
	}

	if (doSweepBack == true && aniOn == true) {
		arcAnimator = new AppleAnimator(1000, 50);
		arcAnimation = new AppleAnimation(mPercentOld, mPercent, arcAnimate);
		arcAnimator.addAnimation(arcAnimation);
		arcAnimator.start();
	}
	else {
		drawPercent(mPercent);
	}
	
	drawPill(yearPercent);
	
	drawText();	
		
	// reset the timer
	startUpdateTimer();		
}



function arcAnimate(animation, current, start, finish)
{
	drawPercent(current); 
}



/*
Drawing functions
*/

function drawText()
{
	setText('currentMonth', fetchIPrefLongMonthName (gCurrentYear, gCurrentMonth));
}



function drawPercent(inPercent)
{
	var canvas = document.getElementById('canvas');
	var context = canvas.getContext('2d');
	
	var mRed = ( ( inPercent - 0.5 ) * 2 ) ;
	var mGreen = (( 1 - inPercent)  *2 ) -1 ;
	var mBlue = ( 1 - ( ( inPercent - 0.5 ) * 2 ) ) ;	
	var mAplha = 0.5;
	
    context.clearRect(0, 0, mHsize, mVsize);
          	
	context.save();
	context.translate(mHcen, mVcen);
	
	context.beginPath();
	context.moveTo(0, 0);
	if (countUp) {
		context.arc(0, 0, arcBig, clockAngle(0), clockAngle(inPercent), false);
		context.arc(0, 0, arcSmall, clockAngle(inPercent), clockAngle(0), true);
	}
	else {
		context.arc(0, 0, arcBig, clockAngle(1), clockAngle(inPercent), true);
		context.arc(0, 0, arcSmall, clockAngle(inPercent), clockAngle(1), false);
	}
	context.closePath();
	context.setFillColor(mRed, mGreen, mBlue, mAplha);	
	context.fill();

	context.restore();
}



function drawTicks()
{
	var canvas = document.getElementById('tickscanvas');
	var context = canvas.getContext('2d');

    context.clearRect(0, 0, mHsize, mVsize);
     
    if (dayTicks == true) {
    	context.drawImage(mTicks, 0, 0, mHsize, mVsize);
    }
    
    if (percentTicks == true) {
		context.drawImage(mPer, 0, 0, mHsize, mVsize);
	}
}



function drawPill(mYearPercent)
{
	var canvas = document.getElementById('pillcanvas');
	var context = canvas.getContext('2d');
	
	var mRed2 = ( ( mYearPercent - 0.5 ) * 2 ) ;
	var mGreen2 = (( 1 - mYearPercent)  *2 ) -1 ;
	var mBlue2 = ( 1 - ( ( mYearPercent - 0.5 ) * 2 ) ) ;
	
	var mAplha = 0.5;

    context.clearRect(0, 0, mHsize, mVsize);
	
	context.drawImage(mPillUnder, 0, 0, mHsize, mVsize);
		
	context.save();
	context.translate(mHcen, mVcen);
	context.beginPath();
	context.moveTo(0, pillDist);
	context.lineTo(pillCen, pillDist);
	context.arc(pillCen, 0, pillDist, clockAngle(0), clockAngle(180), true);
	context.lineTo(pillCen, -pillDist);
	context.lineTo(-pillCen, -pillDist);
	context.arc(-pillCen, 0, pillDist, clockAngle(180), clockAngle(0), true);
	context.lineTo(-pillCen, pillDist);
	context.lineTo(0, pillDist);
	context.closePath();
	context.setFillColor(mRed2, mGreen2, mBlue2, mAplha);	
	context.fill();
		
	context.restore();
	
	context.drawImage(mPill, 0, 0, mHsize, mVsize);
}



/*
support routines
*/

function clockAngle(inPercent)
//given a percentage, returns the necessary radian for the clock display
{
	var mRadian = percentToRadian(inPercent);
	var mAngle = mRadian - ( Math.PI /2 );
	return mAngle;
}



function percentToRadian(inPercent)
//given a percentage, returns radians
{
	var mRadian = inPercent * 2 * Math.PI;
	return(mRadian);
}



function setText(id, text)
{
	obj = document.getElementById(id);
	obj.innerText = text;
}



function calculateNumberOfDaysInMonth(inYear, inMonth)
{
	var returnValue;
	var monthOfInterestYear, monthOfInterestMonth;
	var aTrialDate;
	
	aTrialDate = new Date (inYear, inMonth, 1);
	for (var i = 27;   i < 32;   i++)
	{
		aTrialDate.setDate (i);
		if (aTrialDate.getMonth () != inMonth)
			break;
		else
			returnValue = i;
	}
	
	return returnValue;
}



function fetchIPrefLongMonthName(inYear, inMonthIndex) // returns string
// to do - add in support for international prefs
{
	var returnString = 'Unknown';
	switch (inMonthIndex) {
		case 0:    returnString = 'January';		break;
		case 1:    returnString = 'February';		break;
		case 2:    returnString = 'March';			break;
		case 3:    returnString = 'April';			break;
		case 4:    returnString = 'May';			break;
		case 5:    returnString = 'June';			break;
		case 6:    returnString = 'July';			break;
		case 7:    returnString = 'August';			break;
		case 8:    returnString = 'September';		break;
		case 9:    returnString = 'October';		break;
		case 10:   returnString = 'November';		break;
		case 11:   returnString = 'December';		break;
	}
	return returnString;
}



function countDirectionChanged(selection)
{
	var savePref;
	
	if (document.getElementById('count-popup').value == 2 ) {
		countUp = false;
		savePref = 'countDown';
	}
	else {
		countUp = true;
		savePref = 'countUp';
	}
	
	if (window.widget) {
		widget.setPreferenceForKey(savePref,createkey('pref_countUp'));
	}
}



function updateFrequencyChanged()
{
	var savePref;
	
	if (document.getElementById('update-popup').value == 2 ) {
		updateHourly = false;
		savePref = 'daily';
	}
	else {
		updateHourly = true;
		savePref = 'hourly';
	}
	
	if (window.widget) {
		widget.setPreferenceForKey(savePref,createkey('pref_updateFreq'));
	}
}



function changePercentTicks(selection)
{
	var savePref;
	
	if (selection.checked) {
		percentTicks = true;
		savePref = 'on';
	}
	else {
		percentTicks = false;
		savePref = 'off';
	}
	
	if (window.widget) {
		widget.setPreferenceForKey(savePref, createkey('pref_percentTicks'));
	}
}



function changeDayTicks(selection)
{
	var savePref;
	
	if (selection.checked) {
		dayTicks = true;
		savePref = 'on';
	}
	else {
		dayTicks = false;
		savePref = 'off';
	}
	
	if (window.widget) {
		widget.setPreferenceForKey(savePref, createkey('pref_dayTicks'));
	}
}



function changeAinmate(selection)
{
	var savePref;
	
	if (selection.checked) {
		aniOn = true;
		savePref = 'on';
	}
	else {
		aniOn = false;
		savePref = 'off';
	}
	
	if (window.widget) {
		widget.setPreferenceForKey(savePref, createkey('pref_aniOn'));
	}
}



function createkey(key)
{
	return widget.identifier + '-' + key;
}



function linkURL()
{
	if (window.widget) {
		widget.openURL(gWebSite);
		hidePrefs();
	}

}
